@@ -22,6 +22,27 @@ def masquerade_class(name):
     return decorator
 
 
+def apply_async_wrappers(instance, async_methods, target):
+    for method_name in async_methods:
+        def make_wrapper(method_name):
+            async def wrapper(*args, **kwargs):
+                return getattr(target, method_name)(*args, **kwargs)
+            return wrapper
+
+        setattr(instance, method_name, make_wrapper(method_name))
+
+
+def apply_chaining_wrappers(instance, chaining_methods, target):
+    for method_name in chaining_methods:
+        def make_wrapper(method_name):
+            def wrapper(*args, **kwargs):
+                getattr(target, method_name)(*args, **kwargs)
+                return instance
+            return wrapper
+
+        setattr(instance, method_name, make_wrapper(method_name))
+
+
 @masquerade_class('motor.motor_asyncio.AsyncIOMotorCursor')
 class AsyncCursor():
     PROXIED_CURSOR_CHAINING_METHODS = [
@@ -45,14 +66,11 @@ class AsyncCursor():
     def __init__(self, cursor):
         self.__cursor = cursor
 
-        for method_name in self.PROXIED_CURSOR_CHAINING_METHODS:
-            def make_wrapper(method_name):
-                def wrapper(*args, **kwargs):
-                    getattr(self.__cursor, method_name)(*args, **kwargs)
-                    return self
-                return wrapper
-
-            setattr(self, method_name, make_wrapper(method_name))
+        apply_chaining_wrappers(
+            self,
+            self.PROXIED_CURSOR_CHAINING_METHODS,
+            self.__cursor,
+        )
 
     def __getattr__(self, name):
         return getattr(self.__cursor, name)
@@ -73,24 +91,31 @@ class AsyncCursor():
 @masquerade_class('motor.motor_asyncio.AsyncIOMotorCollection')
 class AsyncMongoMockCollection():
     ASYNC_METHODS = [
+        'bulk_write',
         'count_documents',
-        'count',
+        'count',  # deprecated
         'create_index',
         'create_indexes',
         'delete_many',
         'delete_one',
+        'drop_index',
+        'drop_indexes',
         'drop',
         'ensure_index',
         'estimated_document_count',
-        'find_and_modify',
+        'find_and_modify',  # deprecated
         'find_one_and_delete',
         'find_one_and_replace',
         'find_one_and_update',
         'find_one',
         'index_information',
+        'inline_map_reduce',
         'insert_many',
         'insert_one',
         'map_reduce',
+        'options',
+        'reindex',
+        'rename',
         'replace_one',
         'save',
         'update_many',
@@ -99,14 +124,7 @@ class AsyncMongoMockCollection():
 
     def __init__(self, collection):
         self.__collection = collection
-
-        for method_name in self.ASYNC_METHODS:
-            def make_wrapper(method_name):
-                async def wrapper(*args, **kwargs):
-                    return getattr(self.__collection, method_name)(*args, **kwargs)
-                return wrapper
-
-            setattr(self, method_name, make_wrapper(method_name))
+        apply_async_wrappers(self, self.ASYNC_METHODS, self.__collection)
 
     def __getattr__(self, name):
         return getattr(self.__collection, name)
@@ -120,10 +138,28 @@ class AsyncMongoMockCollection():
 
 @masquerade_class('motor.motor_asyncio.AsyncIOMotorDatabase')
 class AsyncMongoMockDatabase():
+    ASYNC_METHODS = [
+        'create_collection',
+        'dereference',
+        'drop_collection',
+        'list_collection_names',
+        'validate_collection',
+    ]
+
     def __init__(self, database, mock_build_info=None):
         self.__database = database
-        self.__collections = {}
         self.__build_info = mock_build_info or {'ok': 1.0, 'version': '5.0.5'}
+        apply_async_wrappers(self, self.ASYNC_METHODS, self.__database)
+
+    def get_collection(self, *args, **kwargs):
+        return AsyncMongoMockCollection(
+            _patch_collection_internals(
+                self.__database.get_collection(*args, **kwargs),
+            ),
+        )
+
+    def aggregate(self, *args, **kwargs) -> AsyncCursor:
+        return AsyncCursor(self.__database.aggregate(*args, **kwargs))
 
     async def command(self, *args, **kwargs):
         try:
@@ -134,30 +170,40 @@ class AsyncMongoMockDatabase():
             raise
 
     def __getitem__(self, name):
-        return getattr(self, name)
+        return self.get_collection(name)
 
     def __getattr__(self, name):
-        if name not in self.__collections:
-            self.__collections[name] = AsyncMongoMockCollection(
-                _patch_collection_internals(self.__database[name]),
-            )
-        return self.__collections[name]
+        if name in dir(self.__database):
+            return getattr(self.__database, name)
+
+        return self.get_collection(name)
 
 
 @masquerade_class('motor.motor_asyncio.AsyncIOMotorClient')
 class AsyncMongoMockClient():
+    ASYNC_METHODS = [
+        'drop_database',
+        'list_database_names',
+        'list_databases',
+        'server_info',
+    ]
+
     def __init__(self, *args, mock_build_info=None, **kwargs):
         self.__client = MongoClient(*args, **kwargs)
-        self.__databases = {}
         self.__build_info = mock_build_info
+        apply_async_wrappers(self, self.ASYNC_METHODS, self.__client)
+
+    def get_database(self, *args, **kwargs):
+        return AsyncMongoMockDatabase(
+            self.__client.get_database(*args, **kwargs),
+            mock_build_info=self.__build_info,
+        )
 
     def __getitem__(self, name):
-        return getattr(self, name)
+        return self.get_database(name)
 
     def __getattr__(self, name):
-        if name not in self.__databases:
-            self.__databases[name] = AsyncMongoMockDatabase(
-                self.__client[name],
-                mock_build_info=self.__build_info,
-            )
-        return self.__databases[name]
+        if name in dir(self.__client):
+            return getattr(self.__client, name)
+
+        return self.get_database(name)
